"""
Copyright (C) 2018 Alexandre Fioravante de Siqueira

This file is part of 'XXXXXX - Supplementary Material'.

'XXXXXX - Supplementary Material'
is free software: you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the
Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

'XXXXXX - Supplementary Material'
is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with 'XXXXXX -
Supplementary Material'. If not, see <http://www.gnu.org/licenses/>.
"""
from itertools import combinations
from mpl_toolkits.axes_grid1 import host_subplot
from pathlib import Path
from string import ascii_uppercase
import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import mpl_toolkits.axisartist as mpl_aa
import numpy as np
import os
import pandas as pd
import warnings

# Setting up the figures appearance.
plt.rcParams['font.family'] = 'monospace'
plt.rcParams['font.size'] = 30
plt.rcParams['axes.labelsize'] = plt.rcParams['font.size']
plt.rcParams['axes.titlesize'] = 1.2 * plt.rcParams['font.size']
plt.rcParams['legend.fontsize'] = plt.rcParams['font.size']
plt.rcParams['xtick.labelsize'] = plt.rcParams['font.size']
plt.rcParams['ytick.labelsize'] = plt.rcParams['font.size']

# Defining folders.
BASE_PATH = Path('..')
FOLDER_DATA = BASE_PATH/'data'/'02-machado_2020_sample'

# Defining misinformation and disinformation (M&D) categories.
MD_CAT_1 = list(ascii_uppercase[:9])  # subcategories A to I
MD_CAT_2 = list(ascii_uppercase[9:11])  # subcategories J to K
MD_CAT_3 = list(ascii_uppercase[11:12])  # subcategory L
MD_CAT_4 = list(ascii_uppercase[12:13])  # subcategory M
MD_CAT_5 = list(ascii_uppercase[13:14])  # subcategory N
MD_CAT_6 = list(ascii_uppercase[14:17])  # subcategories O to Q

# Defining some helping variables.
OFFSET = -15

LINE_WIDTH = 7
SCATTER_SIZE = 25

SAVE_FIG_FORMAT = '.eps'

MIN_SIZE = 25  # defining minimum size for processing a region.
TEST_REGION = 26

# Ignoring warnings.
warnings.filterwarnings('ignore')


def figure_Y():
    sample_main = pd.read_csv(FOLDER_DATA/'machado_2020-triplicate_vacinaautismo_data.csv')
    sample_2nd = pd.read_csv(FOLDER_DATA/'machado_2020-vaccine_md_videos_extra.csv')
    data_with_misinfo = sample_main[sample_main['hasmisinformation'] == 'Y']

    # aggregating data with misinformation from main and second samples.
    data_with_misinfo = pd.concat([data_with_misinfo, sample_2nd])

    # deleting NaN from our sample.
    data_with_misinfo = data_with_misinfo.dropna(subset=['likecount', 'dislikecount', 'commentcount'])

    data_with_misinfo['misinformationquantity'] = data_with_misinfo.apply(
        lambda row: len(row['misinformationcode'].split()),
        axis=1)

    # setting our engagement measure.
    engagement = (data_with_misinfo['likecount'] +
                  data_with_misinfo['dislikecount'] +
                  data_with_misinfo['commentcount']) / data_with_misinfo['viewcount']
    engagement *= 100

    data_duration = pd.to_timedelta(data_with_misinfo['duration'])

    # Fig 01(b).
    plt.figure(figsize=(16, 10))
    plt.scatter(x=engagement,
                y=data_duration.dt.total_seconds().div(60),
                s=(data_with_misinfo['misinformationquantity']**2)*25,
                c=data_with_misinfo['misinformationquantity'],
                cmap='YlOrRd', edgecolor='k')
    plt.yscale('log')
    plt.ylabel('Duration (min)')
    plt.xlabel('Engagement (%)')
    plt.colorbar()
    plt.tight_layout()
    # Checking if the folder 'figures' exists.
    _check_if_folder_exists(folder='./figures')
    plt.savefig('figures/Fig_0Y' + SAVE_FIG_FORMAT, bbox_inches='tight')
    plt.close()

    return None


def figure_X():
    """
    Figure X.
    """
    sample_main = pd.read_csv(FOLDER_DATA / 'machado_2020-triplicate_vacinaautismo_data.csv')
    sample_2nd = pd.read_csv(FOLDER_DATA / 'machado_2020-vaccine_md_videos_extra.csv')
    data_with_misinfo = sample_main[sample_main['hasmisinformation'] == 'Y']

    # aggregating data with misinformation from main and second samples.
    data_with_misinfo = pd.concat([data_with_misinfo, sample_2nd])
    data_wout_misinfo = sample_main[sample_main['hasmisinformation'] == 'N']

    # Replacing NaN with zeros.
    data_with_misinfo = data_with_misinfo.dropna(subset=['likecount', 'dislikecount', 'commentcount'])
    data_wout_misinfo = data_wout_misinfo.dropna(subset=['likecount', 'dislikecount', 'commentcount'])

    # Fig 1(a).
    stats_misinfo = [data_with_misinfo['viewcount'],
                     data_with_misinfo['commentcount'],
                     data_with_misinfo['likecount'],
                     data_with_misinfo['dislikecount']]
    boxes_num = len(stats_misinfo)
    boxes_range = np.arange(len(stats_misinfo))

    pos = boxes_range
    box_colors = ['#ffffb2', '#fecc5c', '#fd8d3c', '#e31a1c']
    flier_props = dict(marker='P', markerfacecolor='#386cb0',
                       markeredgecolor='#386cb0', linestyle='none')

    x_ticks = boxes_range
    x_labels = ['Views', 'Comments', 'Likes', 'Dislikes']

    fig, ax = plt.subplots(figsize=(16, 10))
    box_plot = ax.boxplot(stats_misinfo, flierprops=flier_props,
                          positions=pos, widths=0.18)

    ax.set_xticks(x_ticks)
    ax.set_xticklabels(x_labels)
    ax.set_yscale('log')

    medians = list(range(boxes_num))

    for i in boxes_range:
        box = box_plot['boxes'][i]
        boxX, boxY = [[] for _ in range(2)]
        for j in boxes_range:
            boxX.append(box.get_xdata()[j])
            boxY.append(box.get_ydata()[j])
        box_coords = list(zip(boxX, boxY))
        box_polygon = mpatches.Polygon(box_coords,
                                       facecolor=box_colors[i])
        ax.add_patch(box_polygon)

        med = box_plot['medians'][i]
        medianX, medianY = [[] for _ in range(2)]
        for j in range(2):
            medianX.append(med.get_xdata()[j])
            medianY.append(med.get_ydata()[j])
            plt.plot(medianX, medianY, 'k')
            medians[i] = medianY[0]
        # overplot the sample averages with horizontal alignment
        # in the center of each box.
        plt.plot([np.average(med.get_xdata())],
                 [np.average(stats_misinfo[i])],
                 color='w', marker='P', markeredgecolor='k')
        # annotate the averages on the side.
        aux_mean = str(np.trunc(stats_misinfo[i].mean()).astype('int'))
        plt.annotate(r'$\mu = $' + aux_mean,
                     xy=(np.average(med.get_xdata())+0.15,
                         np.average(stats_misinfo[i])),
                     fontsize=20,
                     color='k')

    ax.set_xlabel('Videos with M&D')

    # Checking if the folder 'figures' exists.
    _check_if_folder_exists(folder='./figures')
    plt.savefig('figures/Fig_0X' + SAVE_FIG_FORMAT, bbox_inches='tight')
    plt.close()

    return None


def _check_if_folder_exists(folder: str) -> None:
    """Auxiliary function. Check if folder exists and create it if necessary."""
    if not os.path.isdir(folder):
        os.mkdir(folder)
    return None


def generate_all_figures():
    import generate_figures

    print('Generating all figures...')

    for function in dir(generate_figures):
        if function.startswith('figure'):
            item = getattr(generate_figures, function)
            if callable(item):
                item()

    return None


if __name__ == '__main__':
    generate_all_figures()
